package org.processmining.plugins.DataConformance.GUI;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.table.TableCellEditor;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.plugins.DataConformance.framework.ReplayableActivity;
import org.processmining.plugins.DataConformance.framework.VariableMatchCost;
import org.processmining.plugins.DataConformance.framework.VariableMatchCosts;

import com.fluxicon.slickerbox.components.SlickerButton;

class VariableMatchCostListener implements ActionListener {
	
	private VariableMatchCostPanel panel;
	
	static final String MOVEUP = "U";
	static final String MOVEDOWN = "D";		
	static final String ADDMATCH = "A";
	static final String REMOVEMATCH = "R";

	public VariableMatchCostListener(VariableMatchCostPanel activityMatchCostPanel) {
		panel = activityMatchCostPanel;
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand() == ADDMATCH)
			panel.variableCostTable.addRow();
		else if (e.getActionCommand() == REMOVEMATCH)
			panel.variableCostTable.removeRow();
		else if (e.getActionCommand() == MOVEUP)
			panel.variableCostTable.moveUp();
		else if (e.getActionCommand() == MOVEDOWN)
			panel.variableCostTable.moveDown();
	}

}

public class VariableMatchCostPanel<E> extends JPanel {
	
	private final class SaveButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent arg0) {
			final JFileChooser fc = new JFileChooser();
			int retVal = fc.showSaveDialog(variableCostTable);
			if (retVal == JFileChooser.APPROVE_OPTION) {
				try {
					ObjectOutputStream os = new ObjectOutputStream(new FileOutputStream(fc.getSelectedFile()));

					os.writeInt(variableCostTable.model.getValues().size());
					for (VariableMatchCostUI v : variableCostTable.model.getValues()) {
						os.writeObject(v.getActivity().getNode() != null ? getNullSafe(v.getActivity().getNode()
								.getLabel()) : NULL_VALUE);
						os.writeObject(v.getActivity().matchedEventClass != null ? getNullSafe(v.getActivity().matchedEventClass
								.getId()) : NULL_VALUE);
						os.writeObject(getNullSafe(v.getVariable().getLogAttribute()));
						os.writeObject(getNullSafe(v.getVariable().getProcessAttribute()));
						os.writeFloat(v.getCostFaultyValue());
						os.writeFloat(v.getCostNotWriting());
					}

					os.close();
				} catch (FileNotFoundException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}

		private String getNullSafe(String value) {
			return value == null ? NULL_VALUE : value;
		}
	}

	private final class LoadButtonListener implements ActionListener {
		private final Map<? extends ReplayableActivity, E> activityMapping;

		private LoadButtonListener(Map<? extends ReplayableActivity, E> activityMapping) {
			this.activityMapping = activityMapping;
		}

		protected Entry<? extends ReplayableActivity, E> findMatchingActivity(
				Map<? extends ReplayableActivity, E> activityMapping, String activityLabel, String activtyEventClass) {
			for (Entry<? extends ReplayableActivity, E> e : activityMapping.entrySet()) {
				if (e.getKey().getLabel().equals(activityLabel)
						&& e.getValue().toString().equals(activtyEventClass)) {
					return e;
				}
			}
			return null;
		}

		public void actionPerformed(ActionEvent arg0) {
			final JFileChooser fc = new JFileChooser();
			int retVal = fc.showOpenDialog(variableCostTable);
			if (retVal == JFileChooser.APPROVE_OPTION) {
				try {
					ObjectInputStream is = new ObjectInputStream(new FileInputStream(fc.getSelectedFile()));
					int tableSize = is.readInt();
																	
					Vector<VariableMatchCostUI> values = new Vector<VariableMatchCostUI>(tableSize);

					while (values.size() < tableSize) {
						VariableMatchCostUI v = new VariableMatchCostUI();

						String activityLabel = (String) is.readObject();
						String activityEventClass = (String) is.readObject();

						Entry<? extends ReplayableActivity, E> matchingActivity = findMatchingActivity(
								activityMapping, activityLabel, activityEventClass);
						if (matchingActivity == null) {
							v.setActivity(MatchingActivity.ALLFLEXNODES);
						} else {
							v.setActivity((MatchingActivity<? extends ReplayableActivity, XEventClass>) new MatchingActivity<ReplayableActivity, E>(
									matchingActivity.getKey(), matchingActivity.getValue()));
						}

						String varLog = (String) is.readObject();
						String varProcess = (String) is.readObject();

						if (NULL_VALUE.equals(varLog) && NULL_VALUE.equals(varProcess)) {
							v.setVariable(MatchingXAttribute.ALLATTRIBUTES);
						} else {
							v.setVariable(new MatchingXAttribute(varLog, varProcess));
						}

						float costFaultyValue = is.readFloat();
						float costNotWriting = is.readFloat();
						v.setCostFaultyValue(costFaultyValue);
						v.setCostNotWriting(costNotWriting);
						values.add(v);
					}
					is.close();
					VariableMatchingTableModel tableModel = new VariableMatchingTableModel(values);
					TableCellEditor cellEditor1 = variableCostTable.getColumnModel().getColumn(0).getCellEditor();
					TableCellEditor cellEditor2 = variableCostTable.getColumnModel().getColumn(1).getCellEditor();
					variableCostTable.setModel(tableModel);
					variableCostTable.model = tableModel;
					variableCostTable.getColumnModel().getColumn(0).setCellEditor(cellEditor1);
					variableCostTable.getColumnModel().getColumn(1).setCellEditor(cellEditor2);						
				} catch (FileNotFoundException e) {
					e.printStackTrace();
				} catch (IOException e) {
					e.printStackTrace();
				} catch (ClassNotFoundException e) {
					e.printStackTrace();
				}
			}
		}
	}

	private static final String NULL_VALUE = "NULL";

	private JPanel eastPanel = new JPanel();
	private SlickerButton addButton = new SlickerButton("Add");
	private SlickerButton removeButton = new SlickerButton("Remove");
	private SlickerButton moveUpButton = new SlickerButton("Move up");
	private SlickerButton moveDownButton = new SlickerButton("Move down");
	private SlickerButton saveButton = new SlickerButton("Save");
	private SlickerButton loadButton = new SlickerButton("Load");
	VariableMatchingCostTable<E> variableCostTable = null;
	private VariableMatchCostListener listener = new VariableMatchCostListener(this);
	private Set<? extends ReplayableActivity> activitySet;
	private Set<String> variableSet;

	public VariableMatchCostPanel(final Map<? extends ReplayableActivity, E> activityMapping,
			Map<String, String> variableMapping) {
		this.setLayout(new BorderLayout());
		variableCostTable = new VariableMatchingCostTable<E>(activityMapping, variableMapping);
		activitySet = activityMapping.keySet();
		variableSet = variableMapping.keySet();
		JScrollPane jsp = new JScrollPane(variableCostTable);
		jsp.setPreferredSize(new Dimension(122, 187));
		eastPanel.add(addButton);
		addButton.addActionListener(listener);
		addButton.setActionCommand(VariableMatchCostListener.ADDMATCH);
		eastPanel.add(removeButton);
		removeButton.addActionListener(listener);
		removeButton.setActionCommand(VariableMatchCostListener.REMOVEMATCH);
		eastPanel.add(moveUpButton);
		moveUpButton.addActionListener(listener);
		moveUpButton.setActionCommand(VariableMatchCostListener.MOVEUP);
		eastPanel.add(moveDownButton);
		moveDownButton.addActionListener(listener);
		moveDownButton.setActionCommand(VariableMatchCostListener.MOVEDOWN);
		eastPanel.add(saveButton);
		saveButton.addActionListener(new SaveButtonListener());
		eastPanel.add(loadButton);
		loadButton.addActionListener(new LoadButtonListener(activityMapping));
		this.add(new JLabel("Earlier entries have precedence over later entries."), BorderLayout.NORTH);
		this.add(jsp, BorderLayout.CENTER);
		this.add(eastPanel, BorderLayout.SOUTH);
	}

	public VariableMatchCosts getCosts() {
		List<VariableMatchCostUI> values = variableCostTable.model.getValues();
		Vector<VariableMatchCost> retValue = new Vector<VariableMatchCost>(values.size());
		for (VariableMatchCostUI value : values) {
			VariableMatchCost elem = new VariableMatchCost();
			if (value.getActivity().getNode() != null)
				elem.setActivity((value.getActivity().getNode()).getLabel());
			if (value.getVariable().getProcessAttribute() != null)
				elem.setVariable(value.getVariable().getProcessAttribute());
			elem.setCostFaultyValue(value.getCostFaultyValue());
			elem.setCostNotWriting(value.getCostNotWriting());
			elem.setFinal(value.isFinal());
			retValue.add(elem);
		}
		return new VariableMatchCosts(retValue, activitySet, variableSet);
	}

}
