package org.processmining.plugins.DataConformance.framework;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

public class ActivityMatchCosts<E>
{
	private final Map<String, Float> moveInTheProcessCost=Collections.synchronizedMap(new HashMap<String, Float>());
	private final Map<String, Float> moveInTheLogCost=Collections.synchronizedMap(new HashMap<String, Float>());
	private float lowestCost=Float.MAX_VALUE;
	private float highestCost=Float.MIN_VALUE;
	
	public ActivityMatchCosts(Set<? extends ReplayableActivity> activities, Map<? extends ReplayableActivity, E> activityMapping, List<ActivityMatchCost> activityCosts) 
	{
		float cost = 0;
		
		for(ReplayableActivity activity : activities) {
			for(ActivityMatchCost activityCost : activityCosts) {
				if (activityCost.isAllProcessActivities() || activity.equals(activityCost.getProcessActivity())) {
					cost=activityCost.getCost();
					moveInTheProcessCost.put(activity.getLabel(), cost);
					break;	
				}
			}
			if (cost<lowestCost)
				lowestCost=cost;
			if (cost>highestCost)
				highestCost=cost;
		}
		
		//Costs for moving only in the log
		for(Entry<? extends ReplayableActivity, E> eventClass : activityMapping.entrySet()) {
			for(ActivityMatchCost activityCost : activityCosts) {
				if (activityCost.isAllEvents() || eventClass.getValue().equals(activityCost.getEventClass())) {
					cost=activityCost.getCost();
					moveInTheLogCost.put(eventClass.getKey().getLabel(), cost);
					break;	
				}
			}
			if (cost<lowestCost)
				lowestCost=cost;
			if (cost>highestCost)
				highestCost=cost;
			
		}
	}
	
	public float lowestCost()
	{
		return lowestCost;
	}
	
	public float highestCost() {
		return highestCost;
	}

	public float costMoveOnlyInProcess(String activity) 
	{
		assert(activity!=null);
		Float cost=moveInTheProcessCost.get(activity);
		if (cost==null)
			return moveInTheProcessCost.values().iterator().next();
		//assert(cost!=null);
		return cost;
	}

	public float costMoveOnlyInLog(String activity) {
		assert(activity!=null);
		Float cost=moveInTheLogCost.get(activity);	
		if (cost==null)
			return moveInTheProcessCost.values().iterator().next();		
		//assert(cost!=null);
		return cost;
	}
}
