package org.processmining.plugins.DataConformance.visualization.alignment;

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JComponent;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginLevel;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.plugins.DataConformance.Alignment;
import org.processmining.plugins.DataConformance.ResultReplay;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentView.Layout;
import org.processmining.plugins.balancedconformance.export.XAlignmentConverter;
import org.processmining.plugins.balancedconformance.result.AlignmentCollection;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignment;

import com.google.common.base.Function;
import com.google.common.collect.Iterables;

@Plugin(name = "2 Model-Log Alignment in Presence of Data", level = PluginLevel.PeerReviewed, returnLabels = { "Model-Log Alignment in Presence of Data" }, // 
		returnTypes = { JComponent.class }, parameterLabels = { "Matching Instances" } )
@Visualizer
public class AlignmentCollectionVisualizer {

	private AlignmentView dataAlignmentView;

	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualize(PluginContext context, AlignmentCollection alignments) {
		XAlignmentConverter converter = new XAlignmentConverter();
		if (alignments instanceof ResultReplay) {
			ResultReplay result = (ResultReplay) alignments;
			final Map<String, XTrace> traceMap = buildTraceMap(result);
			XTraceResolver traceResolver = new XTraceResolver() {

				public XTrace getOriginalTrace(String name) {
					return traceMap.get(name);
				}

				public boolean hasOriginalTraces() {
					return true;
				}
			};
			converter.setClassifier(result.getClassifier()).setVariableMapping(result.getVariableMapping());
			return doVisualize(context, traceResolver, convertToXAlignment(alignments, traceResolver, converter));
		} else {
			return doVisualize(context, new XTraceResolver() {

				public XTrace getOriginalTrace(String name) {
					return null;
				}

				public boolean hasOriginalTraces() {
					return false;
				}

			}, convertToXAlignment(alignments, new XTraceResolver() {
				
				public boolean hasOriginalTraces() {
					return false;
				}
				
				public XTrace getOriginalTrace(String name) {
					return null;
				}
			},converter));
		}
	}

	private Iterable<XAlignment> convertToXAlignment(AlignmentCollection alignments,
			final XTraceResolver traceResolver, final XAlignmentConverter converter) {
		return Iterables.transform(alignments.getAlignments(), new Function<Alignment, XAlignment>() {

			public XAlignment apply(Alignment a) {
				return converter.viewAsXAlignment(a, traceResolver.getOriginalTrace(a.getTraceName()));
			}
		});
	}

	public JComponent doVisualize(PluginContext context, final XTraceResolver traceResolver,
			Iterable<XAlignment> alignments) {
		Map<String, Color> activityColorMap = ColorTheme.createColorMap(alignments);
		dataAlignmentView = new AlignmentView(Layout.TWOCOLUMN, context, traceResolver, activityColorMap);
		dataAlignmentView.getListView().addAll(alignments);
		return dataAlignmentView;
	}

	private Map<String, XTrace> buildTraceMap(ResultReplay logReplayResult) {
		HashMap<String, XTrace> traceMap = new HashMap<>();
		for (XTrace trace : logReplayResult.getAlignedLog()) {
			traceMap.put(XConceptExtension.instance().extractName(trace), trace);
		}
		return traceMap;
	}

}
