package org.processmining.plugins.DataConformance.visualization.comparison;

import java.awt.Color;
import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Deque;
import java.util.Iterator;
import java.util.List;
import java.util.Map.Entry;

import org.processmining.datapetrinets.expression.GuardExpression;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.AbstractEvent;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PNWDTransition;
import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.DataConformance.visualization.DataAwareStepTypes;

import com.google.common.collect.ImmutableList;

public class AlignmentComparisonEvent extends AbstractEvent {

	private static final String NOMOVE = ">>";
	
	private final ExecutionStep processStep;
	private final ExecutionStep logStep;
	private final DataAwareStepTypes stepType;
	private List<String> attributes;

	public AlignmentComparisonEvent(ExecutionStep processStep, ExecutionStep logStep, DataAwareStepTypes stepType) {
		this.processStep = processStep;
		this.logStep = logStep;
		this.stepType = stepType;
	}

	public String getTopLabel() {
		if (processStep != null) {
			if (processStep.getActivityObject() != null) {
				if (processStep.getActivityObject() instanceof PNWDTransition) {
					GuardExpression guard = ((PNWDTransition)processStep.getActivityObject()).getGuardExpression();
					if (guard != null) {
						return guard.toString();
					}
				}
			}
		}
		return null;
	}

	public String getBottomLabel() {
		return getAttributes().size() >= 1 ? getAttributes().get(0) : null;
	}

	public String getBottomLabel2() {
		return getAttributes().size() == 2 ? getAttributes().get(1) : (getAttributes().size() > 2 ? getAttributes().get(1).concat("..")
				: null);
	}

	public String getLabel() {
		if (processStep.getActivity() != null) {
			return processStep.getActivity();
		} else if (logStep.getActivity() != null) {
			return logStep.getActivity();
		} else {
			return NOMOVE;
		}
	}

	public Color getWedgeColor() {
		return stepType.getColor();
	}

	private List<String> getAttributes() {
		if (attributes == null) {
			attributes = buildVariableAssignmentList(processStep, logStep, stepType);
		}
		return attributes;
	}
	
	private static ImmutableList<Entry<String, Object>> sortVariablesByName(final ExecutionStep step) {
		return ExecutionStep.DEFAULT_VARIABLE_ORDERING.immutableSortedCopy(step.entrySet());
	}

	private static List<String> buildVariableAssignmentList(final ExecutionStep processStep,
			final ExecutionStep logStep, final DataAwareStepTypes stepType) {

		List<String> attributes;
		
		if (stepType == DataAwareStepTypes.LMGOOD) {
			attributes = new ArrayList<>(processStep.size());
			for (Iterator<Entry<String, Object>> iterator = sortVariablesByName(processStep).iterator(); iterator
					.hasNext();) {
				Entry<String, Object> e = iterator.next();
				Object processValue = e.getValue();
				attributes.add(e.getKey() + ":=" + processValue);
			}
		} else if (stepType == DataAwareStepTypes.LMNOGOOD) {
			attributes = new ArrayList<>(processStep.size());
			Deque<Entry<String, Object>> normalAttributes = new ArrayDeque<>();
			for (Iterator<Entry<String, Object>> iterator = sortVariablesByName(processStep).iterator(); iterator
					.hasNext();) {
				Entry<String, Object> e = iterator.next();
				Object logAttr = logStep.get(e.getKey());
				Object processValue = e.getValue();
				if (logAttr != null && !equalAttributes(logAttr, processValue)) {
					// Wrong
					Object correctValue = processValue;
					Object wrongValue = logStep.get(e.getKey());
					attributes.add(e.getKey() + ":=" + correctValue + '\u2260' + wrongValue);
				} else if (logAttr == null) {
					// Missing
					attributes.add(e.getKey() + ":=" + processValue + '\u2260' + "N/A");
				} else {
					// Normal
					normalAttributes.add(e);
				}
			}
			// Add normal attributes if enough space
			if (attributes.size() < 2 && !normalAttributes.isEmpty()) {
				for (Entry<String, Object> e : normalAttributes) {
					Object processValue = e.getValue();
					attributes.add(e.getKey() + ":=" + processValue);
				}
			}
		} else if (stepType == DataAwareStepTypes.MREAL) {
			attributes = new ArrayList<>(processStep.size());
			for (Iterator<Entry<String, Object>> iterator = sortVariablesByName(processStep).iterator(); iterator
					.hasNext();) {
				Entry<String, Object> e = iterator.next();
				Object logAttr = logStep.get(e.getKey());
				if (logAttr == null) {
					// Missing
					attributes.add(e.getKey() + "=" + e.getValue());
				}
			}
		} else {
			attributes = Collections.emptyList();
		}
		
		return attributes;
	}

	private static boolean equalAttributes(Object logAttr, Object processValue) {
		return logAttr.equals(processValue);
	}

}
