package org.processmining.plugins.DataConformance.visualization.grouping;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutionException;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingWorker;

import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.framework.util.ui.widgets.helper.ProMUIHelper;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceList.ClickListener;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceList.TraceBuilder;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.Event;
import org.processmining.framework.util.ui.widgets.traceview.ProMTraceView.Trace;
import org.processmining.framework.util.ui.widgets.traceview.masterdetail.ProMTraceListMasterDetail;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentCollectionVisualizer;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentQueryPredicate;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentTrace;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentTrace.DeviationsSetting;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentTrace.InvisibleSetting;
import org.processmining.plugins.DataConformance.visualization.alignment.AlignmentView;
import org.processmining.plugins.DataConformance.visualization.alignment.ColorTheme;
import org.processmining.plugins.DataConformance.visualization.grouping.GroupedAlignmentMasterView.GroupedAlignmentInput;
import org.processmining.plugins.DataConformance.visualization.grouping.GroupedAlignments.AlignmentGroup;
import org.processmining.xesalignmentextension.XAlignmentExtension.XAlignment;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.google.common.base.Predicate;

public class GroupedAlignmentMasterDetail extends JPanel {

	private final class SearchActionListener implements ActionListener {

		public void actionPerformed(ActionEvent e) {
			new SwingWorker<GroupedAlignments<XAlignment>, Void>() {

				protected GroupedAlignments<XAlignment> doInBackground() throws Exception {
					GroupedAlignmentsSimpleImpl groupedAlignments = new GroupedAlignmentsSimpleImpl(activityColorMap);

					String query = searchQueryField.getText();

					query = addApproximateMatchingIfSelected(query);
					Predicate<XAlignment> predicate = new AlignmentQueryPredicate(query);

					for (XAlignment a : initialData.getAlignments()) {
						if (predicate.apply(a)) {
							groupedAlignments.add(a);
						}
					}
					return groupedAlignments;
				}

				private String addApproximateMatchingIfSelected(String query) {
					if (approximateMatchesCheckBox.isSelected() && !query.isEmpty() && !(query.startsWith("%") || query.contains("=")
							|| query.contains("<") || query.contains(">") || query.startsWith("fitness"))) {
						query = "%" + query;
					}
					return query;
				}

				protected void done() {
					try {
						replaceResult(get());
					} catch (InterruptedException | ExecutionException e) {
						ProMUIHelper.showErrorMessage(detailView, e.getMessage(), "Could not query alignments");
					}
				}

			}.execute();
		}

	}

	private final class VisualizationOptionsListener implements ActionListener {

		private final JCheckBox deviationsCheckBox;
		private final JCheckBox unobserveavleCheckBox;
		private final JCheckBox colorActivitiesCheckBox;

		private VisualizationOptionsListener(JCheckBox deviationsCheckBox, JCheckBox unobserveavleCheckBox,
				JCheckBox colorActivitiesCheckBox) {
			this.deviationsCheckBox = deviationsCheckBox;
			this.unobserveavleCheckBox = unobserveavleCheckBox;
			this.colorActivitiesCheckBox = colorActivitiesCheckBox;
		}

		public void actionPerformed(ActionEvent e) {
			getMasterView().getMasterList().setTraceBuilder(new TraceBuilder<AlignmentGroup>() {

				public Trace<? extends Event> build(AlignmentGroup a) {
					return a.getRepresentative(
							unobserveavleCheckBox.getModel().isSelected() ? InvisibleSetting.HIDDEN
									: InvisibleSetting.VISIBLE,
							deviationsCheckBox.getModel().isSelected() ? DeviationsSetting.HIGHLIGHTED
									: DeviationsSetting.NORMAL,
							colorActivitiesCheckBox.getModel().isSelected());
				}

			});
			getDetailView().getListView().setTraceBuilder(new TraceBuilder<XAlignment>() {

				public Trace<? extends Event> build(XAlignment a) {
					return new AlignmentTrace(a, activityColorMap,
							unobserveavleCheckBox.getModel().isSelected() ? InvisibleSetting.HIDDEN
									: InvisibleSetting.VISIBLE,
							deviationsCheckBox.getModel().isSelected() ? DeviationsSetting.HIGHLIGHTED
									: DeviationsSetting.NORMAL,
							colorActivitiesCheckBox.getModel().isSelected());
				}

			});
			if (deviationsCheckBox.getModel().isSelected()) {
				getDetailView().getListView().setFixedWedgeLimit(Integer.MAX_VALUE);
			} else {
				getDetailView().getListView().setFixedWedgeLimit(10000);
			}
			getMasterView().getMasterList().repaint();
			getDetailView().getListView().repaint();

		}
	}

	private static final long serialVersionUID = -7681178293685250088L;

	private final ProMTraceListMasterDetail<AlignmentGroup, XAlignment> masterDetailView;
	private final GroupedAlignmentMasterView<XAlignment> masterView;
	private final GroupedAlignmentDetailView detailView;

	private final Map<String, Color> activityColorMap;

	private GroupedAlignments<XAlignment> initialData;

	private ProMTextField searchQueryField;

	private JCheckBox colorActivitiesCheckBox;
	private JCheckBox approximateMatchesCheckBox;

	public GroupedAlignmentMasterDetail(final PluginContext context, final GroupedAlignmentInput<XAlignment> input) {
		super();
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		setOpaque(true);
		setBackground(null);
		setForeground(null);

		initialData = input.groupedResult;
		activityColorMap = input.activityColorMap;

		masterView = new GroupedAlignmentMasterView<>(context, input);
		masterView.setOpaque(true);

		detailView = new GroupedAlignmentDetailView(context, input);
		detailView.setBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK));
		detailView.setOpaque(true);

		masterView.getMasterList().getList().addMouseListener(new MouseAdapter() {

			public void mouseReleased(MouseEvent e) {
				if (e.isPopupTrigger())
					showMenu(e);
			}

			public void mousePressed(MouseEvent e) {
				if (e.isPopupTrigger())
					showMenu(e);
			}

			private void showMenu(MouseEvent e) {
				JPopupMenu menu = new JPopupMenu();
				if (masterView.getMasterList().getList().getSelectedIndex() != -1) {
					final int countSelected = masterView.getMasterList().getList().getSelectedIndices().length;
					final JMenuItem menuItemExport = new JMenuItem(
							String.format("Export %s selected alignment group(s) to ProM", countSelected));
					menuItemExport.addActionListener(new ActionExportAsAlignment(context, masterView));
					menu.add(menuItemExport);

					if (input.traceResolver.hasOriginalTraces()) {
						final JMenuItem menuItemExportLog = new JMenuItem(String
								.format("Export log traces of %s selected alignment group(s) to ProM", countSelected));
						menuItemExportLog.addActionListener(
								new ActionExportOriginalTraces(context, input.traceResolver, masterView));
						menu.add(menuItemExportLog);
					}

					JMenuItem menuItemSaveAsPDF = new JMenuItem("Save as PDF/EMF/EPS");
					menuItemSaveAsPDF
							.addActionListener(new ActionListenerSaveAs(masterView, masterView.getCurrentOrdering()));
					menu.add(menuItemSaveAsPDF);

				}
				menu.show(e.getComponent(), e.getX(), e.getY());
			}

		});

		masterView.getMasterList().addTraceClickListener(new ClickListener<AlignmentGroup>() {

			public void traceMouseDoubleClicked(final AlignmentGroup group, int traceIndex, int eventIndex,
					MouseEvent e) {
				SwingWorker<Collection<XAlignment>, Object> bgWorker = new SwingWorker<Collection<XAlignment>, Object>() {

					@Override
					public Collection<XAlignment> doInBackground() {
						return masterView.getDetailElements(group);
					}

					@Override
					protected void done() {
						try {
							JFrame newFrame = new JFrame(getTitle(group));
							newFrame.add(new AlignmentCollectionVisualizer().doVisualize(context, input.traceResolver,
									get()));
							newFrame.pack();
							newFrame.setVisible(true);
						} catch (HeadlessException e) {
							ProMUIHelper.showErrorMessage(masterDetailView, e.getMessage(), "Error loading details");
						} catch (InterruptedException e) {
							ProMUIHelper.showErrorMessage(masterDetailView, e.getMessage(), "Error loading details");
						} catch (ExecutionException e) {
							ProMUIHelper.showErrorMessage(masterDetailView, e.getMessage(), "Error loading details");
						}
					}

					private String getTitle(AlignmentGroup group) {
						StringBuilder stringBuilder = new StringBuilder();
						if (group.getName() != null) {
							stringBuilder.append(group.getName());
						} else {
							stringBuilder.append("Alignments grouped by same Control-Flow");
						}
						stringBuilder.append(" (#");
						stringBuilder.append(group.size());
						stringBuilder.append(", avg. fitness ");
						stringBuilder.append(NumberFormat.getPercentInstance().format(group.getAverageFitness()));
						return stringBuilder.toString();

					}

				};
				bgWorker.execute();
			}

			public void traceMouseClicked(AlignmentGroup trace, int traceIndex, int eventIndex, MouseEvent e) {
				// No-op
			}

		});

		masterDetailView = new ProMTraceListMasterDetail<>(masterView, detailView);		
		add(masterDetailView);

		@SuppressWarnings("rawtypes")
		final JComboBox sortCbx = SlickerFactory.instance().createComboBox(GroupedAlignments.GroupOrdering.values());
		sortCbx.setMaximumSize(new Dimension(300, 30));
		sortCbx.setSelectedItem(GroupedAlignments.GroupOrdering.COUNT_DESC);
		sortCbx.addActionListener(new ActionListener() {

			@SuppressWarnings("unchecked")
			public void actionPerformed(ActionEvent e) {
				masterView.setCurrentOrdering((Comparator<AlignmentGroup>) sortCbx.getSelectedItem());
			}
		});
		masterDetailView.getMasterView().getMasterList().getToolbar().add(sortCbx);
		
		final JPanel bottomPanel = new JPanel();
		bottomPanel.setOpaque(false);
		bottomPanel.setForeground(null);
		bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.X_AXIS));
		bottomPanel.setBorder(BorderFactory.createMatteBorder(1, 0, 0, 0, Color.BLACK));

		bottomPanel.add(Box.createHorizontalStrut(20));
		

		searchQueryField = new ProMTextField();
		searchQueryField.setHint("Filter by trace/event names or by fitness (fitness>0.5)");
		searchQueryField.getTextField().setToolTipText(
				"<html>" + "Supports a simple query language for filtering alignments by names of events and traces or by their fitness value (e.g., fitness>0.5)."
						+ "<br>Use '~' as 1st character to use a regular expressions, use '%' as 1st character to use a 'contains' query. "
						+ "</html>");
		searchQueryField.setMinimumSize(new Dimension(100, 30));
		searchQueryField.setPreferredSize(new Dimension(500, 30));
		searchQueryField.setMaximumSize(new Dimension(500, 30));

		searchQueryField.addActionListener(new SearchActionListener());
		bottomPanel.add(searchQueryField);

		bottomPanel.add(Box.createHorizontalStrut(5));
		approximateMatchesCheckBox = SlickerFactory.instance().createCheckBox("Sub-string Matches", true);
		approximateMatchesCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		approximateMatchesCheckBox.addActionListener(new SearchActionListener());
		bottomPanel.add(approximateMatchesCheckBox);

		bottomPanel.add(Box.createHorizontalGlue());

		bottomPanel.add(Box.createHorizontalStrut(5));
		colorActivitiesCheckBox = SlickerFactory.instance().createCheckBox("Color Code Activities", false);
		colorActivitiesCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		bottomPanel.add(colorActivitiesCheckBox);

		bottomPanel.add(Box.createHorizontalStrut(20));

		bottomPanel.add(Box.createHorizontalStrut(5));
		final JCheckBox unobserveavleCheckBox = SlickerFactory.instance()
				.createCheckBox("Hide/Show Invisible Activities", false);
		unobserveavleCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		bottomPanel.add(unobserveavleCheckBox);

		bottomPanel.add(Box.createHorizontalStrut(20));

		bottomPanel.add(Box.createHorizontalStrut(5));
		final JCheckBox deviationsCheckBox = SlickerFactory.instance().createCheckBox("Highlight Deviations", false);
		deviationsCheckBox.setAlignmentX(Component.CENTER_ALIGNMENT);
		bottomPanel.add(deviationsCheckBox);

		bottomPanel.add(Box.createHorizontalStrut(20));
		
		@SuppressWarnings("rawtypes")
		final JComboBox colorCbx = SlickerFactory.instance().createComboBox(ColorTheme.values());
		colorCbx.setMaximumSize(new Dimension(200, 30));
		colorCbx.setSelectedItem(ColorTheme.BRIGHT);
		colorCbx.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				setColor((ColorTheme) colorCbx.getSelectedItem());
			}
		});
		bottomPanel.add(colorCbx);

		VisualizationOptionsListener listener = new VisualizationOptionsListener(deviationsCheckBox,
				unobserveavleCheckBox, colorActivitiesCheckBox);
		colorActivitiesCheckBox.addActionListener(listener);
		unobserveavleCheckBox.addActionListener(listener);
		deviationsCheckBox.addActionListener(listener);
		
		add(bottomPanel);
		setColor(ColorTheme.BRIGHT);
		detailView.updateStatistics(input.groupedResult.getAlignments());
	}

	public synchronized void replaceResult(final GroupedAlignments<XAlignment> newResult) {
		masterDetailView.disableDetailUpdates();

		masterView.replace(newResult);
		updateDetailView(newResult);
		detailView.updateStatistics(newResult.getAlignments());

		masterDetailView.enableDetailUpdates();
	}

	private void updateDetailView(final GroupedAlignments<XAlignment> replayResult) {
		final int[] selectedAlignmentGroups = masterView.getMasterList().getList().getSelectedIndices();

		SwingWorker<Collection<XAlignment>, Object> bgWorker = new SwingWorker<Collection<XAlignment>, Object>() {

			@Override
			public Collection<XAlignment> doInBackground() {
				List<XAlignment> detailedAlignments = new ArrayList<>();
				for (int index : selectedAlignmentGroups) {
					final AlignmentGroup alignmentGroup = masterView.getMasterList().getListModel().getElementAt(index);
					detailedAlignments.addAll(replayResult.listAlignments(alignmentGroup));
				}
				return detailedAlignments;
			}

			@Override
			protected void done() {
				try {
					Set<XAlignment> previousSelection = getSelectedAlignments(detailView.getDetailList().getList());
					detailView.getDetailList().getList().clearSelection();
					detailView.getDetailList().clear();
					Collection<XAlignment> elements = get();
					detailView.getDetailList().addAll(elements);
					detailView.getDetailList().sort(detailView.getSortOrder());
					for (int i = 0; i < detailView.getDetailList().getListModel().getSize(); i++) {
						XAlignment alignment = detailView.getDetailList().getListModel().getElementAt(i);
						if (previousSelection.contains(alignment)) {
							detailView.getDetailList().getList().addSelectionInterval(i, i);
						}
					}
				} catch (HeadlessException e) {
					ProMUIHelper.showErrorMessage(masterDetailView, e.getMessage(), "Error loading details");
				} catch (InterruptedException e) {
					ProMUIHelper.showErrorMessage(masterDetailView, e.getMessage(), "Error loading details");
				} catch (ExecutionException e) {
					ProMUIHelper.showErrorMessage(masterDetailView, e.getMessage(), "Error loading details");
				}
			}

		};
		bgWorker.execute();
	}

	private static Set<XAlignment> getSelectedAlignments(@SuppressWarnings("rawtypes") JList jList) {
		Set<XAlignment> selectedValues = new HashSet<>();
		int[] selectedIndices = jList.getSelectedIndices();
		for (int i = 0; i < selectedIndices.length; i++) {
			int index = selectedIndices[i];
			if (index < jList.getModel().getSize()) {
				selectedValues.add(((XAlignment) jList.getModel().getElementAt(index)));
			}

		}
		return selectedValues;
	}

	public void setColor(ColorTheme theme) {
		// Global background for all child non-opaque components
		setBackground(theme.getBackground());
		setForeground(theme.getForeground());

		masterView.setBackground(theme.getBackground());
		detailView.setBackground(theme.getBackground());

		masterView.setForeground(theme.getForeground());
		detailView.setForeground(theme.getForeground());
	}

	public GroupedAlignmentMasterView<XAlignment> getMasterView() {
		return masterView;
	}

	public ProMTraceListMasterDetail<AlignmentGroup, XAlignment> getMasterDetailView() {
		return masterDetailView;
	}

	public AlignmentView getDetailView() {
		return detailView.getAlignmentView();
	}

	public void disableColorCodeActivities() {
		colorActivitiesCheckBox.setSelected(false);
	}

	public void disableSearch() {
		searchQueryField.setEnabled(false);
	}

	public void enableSearch() {
		searchQueryField.setEnabled(true);
	}

}
