package org.processmining.plugins.balancedconformance.controlflow.adapter;

import java.util.Map;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.model.XLog;
import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.models.graphbased.directed.petrinet.InhibitorNet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.ResetInhibitorNet;
import org.processmining.models.graphbased.directed.petrinet.ResetNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.astar.petrinet.impl.AbstractPDelegate;
import org.processmining.plugins.astar.petrinet.impl.PHead;
import org.processmining.plugins.astar.petrinet.impl.PILPTail;
import org.processmining.plugins.balancedconformance.controlflow.override.XPILPDelegate;
import org.processmining.plugins.balancedconformance.dataflow.DataAlignmentAdapter;
import org.processmining.plugins.balancedconformance.dataflow.datastate.DataStateStore;
import org.processmining.plugins.balancedconformance.mapping.LogMapping;
import org.processmining.plugins.balancedconformance.observer.DataConformancePlusObserver;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;

import nl.tue.astar.impl.AbstractAStarThread.StorageHandler;

public abstract class AlignmentAdapterILPAbstractImpl extends AlignmentAbstractAdapter<PILPTail> {
	
	private boolean isKeepCFSearchSpace;
	private boolean isKeepDFSearchSpace;

	public AlignmentAdapterILPAbstractImpl(XLog log, DataPetriNet net, LogMapping logMapping,
			ControlFlowAlignmentConfig c, StorageHandlerFactory<PILPTail> storageFactory, DataConformancePlusObserver dataConformancePlusObserver,
			DataAlignmentAdapter dataAdapter, boolean isKeepCFSearchSpace, boolean isKeepDFSearchSpace) {
		super(log, net, logMapping, c, storageFactory, dataConformancePlusObserver, dataAdapter);
		this.isKeepCFSearchSpace = isKeepCFSearchSpace;
		this.isKeepDFSearchSpace = isKeepDFSearchSpace;
	}

	/*
	 * XPILPDelegate(PetrinetGraph net, XLog log, XEventClasses classes, TransEvClassMapping map,
			Map<Transition, Integer> mapTrans2Cost, Map<XEventClass, Integer> mapEvClass2Cost,
			Map<Transition, Integer> mapSync2Cost, int delta, int threads, Marking... set)
	 */
	
	protected AbstractPDelegate<PILPTail> getPDelegate(PetrinetGraph net, XLog log, XEventClasses classes,
			TransEvClassMapping map, Map<Transition, Integer> mapTrans2Cost, Map<XEventClass, Integer> mapEvClass2Cost,
			int delta, boolean allMarkingsAreFinal, Marking[] finalMarkings) {
		if (net instanceof ResetInhibitorNet) {
			return new XPILPDelegate((ResetInhibitorNet) net, log, classes, map, mapTrans2Cost, mapEvClass2Cost, delta,
					Runtime.getRuntime().availableProcessors() + 1, finalMarkings);
		} else if (net instanceof ResetNet) {
			return new XPILPDelegate((ResetNet) net, log, classes, map, mapTrans2Cost, mapEvClass2Cost, delta, Runtime
					.getRuntime().availableProcessors() + 1, finalMarkings);
		} else if (net instanceof InhibitorNet) {
			return new XPILPDelegate((InhibitorNet) net, log, classes, map, mapTrans2Cost, mapEvClass2Cost, delta,
					Runtime.getRuntime().availableProcessors() + 1, finalMarkings);
		} else 
			return new XPILPDelegate(net, log, classes, map, mapTrans2Cost, mapEvClass2Cost, delta, Runtime
					.getRuntime().availableProcessors() + 1, finalMarkings);

		//throw new UnsupportedOperationException("Unkown type of Petri net " + net.getClass().getSimpleName());
	}

	protected DataStateStore getDataStore() {
		if (!isKeepDFSearchSpace) {
			return storageFactory.newDataStateStore();			
		} else {
			return dataStateStore;
		}
	}

	protected StorageHandler<PHead, PILPTail> getStorageHandler() {
		if (!isKeepCFSearchSpace) {
			return storageFactory.newStorageHandler(delegate);			
		} else {
			return storageHandler;
		}
	}

}