/*
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 */
package org.processmining.plugins.balancedconformance.controlflow.adapter;

import java.util.Map;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.processmining.datapetrinets.DataPetriNet;
import org.processmining.plugins.astar.petrinet.impl.AbstractPDelegate;
import org.processmining.plugins.astar.petrinet.impl.PHead;
import org.processmining.plugins.astar.petrinet.impl.PILPTail;
import org.processmining.plugins.balancedconformance.cache.DataViewBuilder;
import org.processmining.plugins.balancedconformance.controlflow.override.DHead;
import org.processmining.plugins.balancedconformance.controlflow.override.DRecord;
import org.processmining.plugins.balancedconformance.controlflow.override.DataAwareGraphThread;
import org.processmining.plugins.balancedconformance.controlflow.override.DataAwareThread;
import org.processmining.plugins.balancedconformance.controlflow.override.queues.noduplicate.BFDataPriorityQueue;
import org.processmining.plugins.balancedconformance.controlflow.override.queues.noduplicate.DFDataPriorityQueue;
import org.processmining.plugins.balancedconformance.controlflow.override.queues.noduplicate.RandomDataPriorityQueue;
import org.processmining.plugins.balancedconformance.controlflow.resultbuilder.AlignmentResultBuilder;
import org.processmining.plugins.balancedconformance.controlflow.resultbuilder.AlignmentResultBuilderImpl;
import org.processmining.plugins.balancedconformance.dataflow.DataAlignmentAdapter;
import org.processmining.plugins.balancedconformance.dataflow.datastate.DataStateStore;
import org.processmining.plugins.balancedconformance.mapping.LogMapping;
import org.processmining.plugins.balancedconformance.mapping.TraceMapping;
import org.processmining.plugins.balancedconformance.observer.DataConformancePlusObserver;

import it.unimi.dsi.fastutil.Hash.Strategy;
import nl.tue.astar.AStarObserver;
import nl.tue.astar.AStarThread.QueueingModel;
import nl.tue.astar.Tail;
import nl.tue.astar.Trace;
import nl.tue.astar.impl.AbstractAStarThread;
import nl.tue.astar.impl.AbstractAStarThread.StorageHandler;
import nl.tue.astar.util.FastLookupPriorityQueue;

/**
 * A* graph-based (i.e. prunes equal states that already have been visited)
 * search for the best alignment taking data into account.
 * 
 * @author F. Mannhardt
 * 
 */
public class AlignmentAdapterILPGraphImpl extends AlignmentAdapterILPAbstractImpl {

	public AlignmentAdapterILPGraphImpl(XLog log, DataPetriNet net, LogMapping logMapping, ControlFlowAlignmentConfig c,
			StorageHandlerFactory<PILPTail> storageFactory, boolean isKeepCFSearchSpace, boolean isKeepDFSearchSpace,
			DataConformancePlusObserver dataConformancePlusObserver, DataAlignmentAdapter dataAdapter) {
		super(log, net, logMapping, c, storageFactory, dataConformancePlusObserver, dataAdapter, isKeepCFSearchSpace,
				isKeepDFSearchSpace);
	}

	protected AlignmentResultBuilder createResultBuilder(TraceMapping traceMapping,
			Map<XEvent, XEventClass> mapEv2EventClass, Trace linearTrace, AStarObserver observer) {

		StorageHandler<PHead, PILPTail> localStorageHandler = getStorageHandler();
		DataStateStore localDataStateStore = getDataStore();
		traceMapping.setDataStateStore(localDataStateStore);
		
		Strategy<DRecord> strategy = getStateGraphStrategy(traceMapping, dataAdapter);

		DHead initialHead = new DHead(delegate, initMarking, traceMapping.getTrace());
		AbstractAStarThread<PHead, PILPTail> thread = new DataAwareGraphThread<>(delegate, initialHead,
				localStorageHandler, linearTrace, maxNumOfStates, dataAdapter, logMapping, traceMapping,
				mapEv2EventClass, mapEvClass2Cost, dataViewBuilder, activateCache, useOptimizations, sorting,
				getQueueFactory(queueingModel, strategy), localDataStateStore, strategy);

		return new AlignmentResultBuilderImpl((DataAwareThread<PHead, ? extends Tail>) thread, linearTrace,
				traceMapping, canceller, delegate, mapEv2EventClass, mapEvClass2Cost, timeLimit, observer);
	}

	private PriorityQueueFactory getQueueFactory(QueueingModel queueingModel, final Strategy<DRecord> strategy) {
		switch (queueingModel) {
			case BREADTHFIRST :
				return new PriorityQueueFactory() {
					public FastLookupPriorityQueue newInstance(DataViewBuilder dataViewBuilder,
							AbstractPDelegate<? extends Tail> delegate) {
						return new BFDataPriorityQueue(1024 * 2, strategy, delegate);
					}

				};

			case DEPTHFIRST :
				return new PriorityQueueFactory() {

					public FastLookupPriorityQueue newInstance(DataViewBuilder dataViewBuilder,
							AbstractPDelegate<? extends Tail> delegate) {
						return new DFDataPriorityQueue(1024 * 2, strategy, delegate);
					}

				};

			case RANDOM :
				return new PriorityQueueFactory() {

					public FastLookupPriorityQueue newInstance(DataViewBuilder dataViewBuilder,
							AbstractPDelegate<? extends Tail> delegate) {
						return new RandomDataPriorityQueue(1024 * 2, strategy, delegate);
					}

				};

			default :
				// Fallback
				throw new UnsupportedOperationException("Not supported " + queueingModel);
		}
	}

}