package org.processmining.plugins.balancedconformance.controlflow.override;

public final class PackUtils {

	private PackUtils() {
		super();
	}

	private static final int UINT16_MASK = 0xFFFF;
	public static final int MAX_UINT16_PACKABLE = UINT16_MASK;

	public static int unpackUInt16Left(int packedValue) {
		return packedValue >>> 16;
	}

	public static int unpackUInt16Right(int packedValue) {
		return packedValue & UINT16_MASK;
	}

	public static int packUInt16(int right, int left) {
		assert right <= MAX_UINT16_PACKABLE && right >= 0 : "Cannot pack " + right;
		assert left <= MAX_UINT16_PACKABLE && left >= 0 : "Cannot pack " + left;
		return (left << 16 | right & UINT16_MASK);
	}

	public static boolean isPackableUInt16(int right, int left) {
		return right <= PackUtils.MAX_UINT16_PACKABLE && left <= PackUtils.MAX_UINT16_PACKABLE && right >= 0
				&& left >= 0;
	}

	private static final long UINT21_MASK = 0x1FFFFFl;
	public static final int MAX_UINT21_PACKABLE = (int) UINT21_MASK;

	public static long packUInt21(int right, int middle, int left) {
		assert right <= MAX_UINT21_PACKABLE && right >= 0 : "Cannot pack " + right;
		assert middle <= MAX_UINT21_PACKABLE && middle >= 0 : "Cannot pack " + middle;
		assert left <= MAX_UINT21_PACKABLE && left >= 0 : "Cannot pack " + left;
		return (left & UINT21_MASK) << 21 | (middle & UINT21_MASK) << 42 | right & UINT21_MASK;
	}

	public static int unpackUInt21Right(long packedValue) {
		return (int) (packedValue & UINT21_MASK);
	}
	
	public static long changeUInt21Right(long packedValue, int middle) {
		packedValue &= ~(UINT21_MASK);
		return packedValue | (middle & UINT21_MASK);
	}

	public static int unpackUInt21Middle(long packedValue) {
		return (int) ((packedValue >>> 42) & UINT21_MASK);
	}
	
	public static long changeUInt21Middle(long packedValue, int middle) {
		packedValue &= ~(UINT21_MASK << 42);
		return packedValue | (middle & UINT21_MASK) << 42;
	}

	public static int unpackUInt21Left(long packedValue) {
		return (int) ((packedValue >>> 21) & UINT21_MASK);
	}
	
	public static long changeUInt21Left(long packedValue, int middle) {
		packedValue &= ~(UINT21_MASK << 21);
		return packedValue | (middle & UINT21_MASK) << 21;
	}

	public static boolean isPackableUInt21(int right, int middle, int left) {
		return right < PackUtils.MAX_UINT21_PACKABLE && middle < PackUtils.MAX_UINT21_PACKABLE
				&& left < PackUtils.MAX_UINT21_PACKABLE && right >= 0 && middle >= 0 && left >= 0;
	}

	private static final long UINT32_MASK = 0xffffffffL;
	public static final long MAX_UINT32_PACKABLE = UINT32_MASK;

	public static long packUInt32(long right, long left) {
		return (right << 32) | (left & UINT32_MASK);
	}

	public static long unpackUInt32Left(long packedValue) {
		return packedValue & UINT32_MASK;
	}
	
	public static long unpackUInt32Right(long packedValue) {
		return packedValue >>> 32 & UINT32_MASK;
	}

	public static long changeUInt32Left(long packedValue, long left) {
		packedValue &= ~(UINT32_MASK);
		return packedValue | (left & UINT32_MASK);
	}
	
	public static long changeUInt32Right(long packedValue, long right) {
		packedValue &= ~(UINT32_MASK << 32);
		return packedValue | (right & UINT32_MASK) << 32;
	}

}
