package org.processmining.plugins.balancedconformance.controlflow.override;

import gnu.trove.iterator.TShortObjectIterator;
import gnu.trove.list.TIntList;
import gnu.trove.list.array.TIntArrayList;

import java.util.Map;

import nl.tue.astar.impl.DijkstraTail;
import nl.tue.astar.util.ShortShortMultiset;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClasses;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.models.graphbased.directed.petrinet.InhibitorNet;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.ResetInhibitorNet;
import org.processmining.models.graphbased.directed.petrinet.ResetNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;
import org.processmining.plugins.astar.petrinet.impl.PDelegate;
import org.processmining.plugins.astar.petrinet.impl.PHeadCompressor;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;

public final class XPDelegate extends PDelegate {

	private Transition[] transArray;
	private short[][] trans2InputArray;
	private short[][] trans2OutputArray;

	public XPDelegate(InhibitorNet net, XLog log, XEventClasses classes, TransEvClassMapping map,
			Map<Transition, Integer> mapTrans2Cost, Map<XEventClass, Integer> mapEvClass2Cost, int delta,
			boolean allMarkingsAreFinal, Marking... set) {
		super(net, log, classes, map, mapTrans2Cost, mapEvClass2Cost, delta, allMarkingsAreFinal, set);
		initalize();
	}

	public XPDelegate(Petrinet net, XLog log, XEventClasses classes, TransEvClassMapping map,
			Map<Transition, Integer> mapTrans2Cost, Map<XEventClass, Integer> mapEvClass2Cost, int delta,
			boolean allMarkingsAreFinal, Marking... set) {
		super(net, log, classes, map, mapTrans2Cost, mapEvClass2Cost, delta, allMarkingsAreFinal, set);
		initalize();
	}

	public XPDelegate(ResetInhibitorNet net, XLog log, XEventClasses classes, TransEvClassMapping map,
			Map<Transition, Integer> mapTrans2Cost, Map<XEventClass, Integer> mapEvClass2Cost, int delta,
			boolean allMarkingsAreFinal, Marking... set) {
		super(net, log, classes, map, mapTrans2Cost, mapEvClass2Cost, delta, allMarkingsAreFinal, set);
		initalize();
	}

	public XPDelegate(ResetNet net, XLog log, XEventClasses classes, TransEvClassMapping map,
			Map<Transition, Integer> mapTrans2Cost, Map<XEventClass, Integer> mapEvClass2Cost, int delta,
			boolean allMarkingsAreFinal, Marking... set) {
		super(net, log, classes, map, mapTrans2Cost, mapEvClass2Cost, delta, allMarkingsAreFinal, set);
		initalize();
	}

	private void initalize() {
		transArray = new Transition[int2trans.size()];
		convertToArray(int2trans.iterator(), transArray);

		trans2InputArray = new short[transIndex2input.size()][];
		convertToArray(transIndex2input.iterator(), trans2InputArray);

		trans2OutputArray = new short[transIndex2output.size()][];
		convertToArray(transIndex2output.iterator(), trans2OutputArray);
	}

	private static <T> void convertToArray(TShortObjectIterator<T> iter, T[] a) {
		while (iter.hasNext()) {
			iter.advance();
			a[iter.key()] = iter.value();
		}
	}

	@Override
	protected PHeadCompressor<DijkstraTail> constructHeadCompressor(short places2, short activities2) {
		return new DHeadCompressor<DijkstraTail>(places, activities);
	}

	@Override
	public boolean isEnabled(int transition, ShortShortMultiset marking) {
		// Overriden for efficiency, only check marking when tokens are needed or inhibited
		short[] needed = trans2InputArray[transition];
		short[] markingVals = marking.getInternalValues();

		for (short i = places; i-- > 0;) {
			short tokensNeeded = needed[i];
			if (tokensNeeded != 0) {
				short tokensAvail = markingVals[i];
				if (tokensAvail < tokensNeeded) {
					return false;
				} else if (tokensNeeded == INHIBITED) { // implies "tokensAvail > 0"
					return false;
				}
			}
		}
		return true;
	}

	@Override
	public TIntList getEnabledTransitionsChangingMarking(ShortShortMultiset marking) {
		// Overridden to also all enabled transitions, as those might have an effect on the 'data state'
		TIntList list = new TIntArrayList(transitions);
		for (short t = 0; t < transitions; t++) {
			if (isEnabled(t, marking)) {
				list.add(t);
			}
		}
		return list;
	}

	@Override
	public Transition getTransition(short i) {
		return transArray[i];
	}

	@Override
	public short[] getInputOf(short transition) {
		return trans2InputArray[transition];
	}

	@Override
	public short[] getOutputOf(short transition) {
		return trans2OutputArray[transition];
	}

	public XTrace getTrace(int t) {
		throw new UnsupportedOperationException("This implementation is un-aware of the full XLog!");
	}

	public int getActivityOf(int trace, int event) {
		throw new UnsupportedOperationException("This implementation is un-aware of the full XLog!");
	}

}