package org.processmining.plugins.balancedconformance.controlflow.override.storage;

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import nl.tue.astar.AStarException;
import nl.tue.astar.Record;
import nl.tue.astar.Tail;
import nl.tue.astar.impl.State;
import nl.tue.astar.impl.memefficient.MemoryEfficientAStarAlgorithm;
import nl.tue.astar.impl.memefficient.MemoryEfficientStorageHandler;
import nl.tue.storage.StorageException;
import nl.tue.storage.impl.CompressedStoreHashSetImpl.Result;

import org.processmining.plugins.astar.petrinet.impl.PHead;

public final class MemoryEfficientStorageHandlerRWLock<T extends Tail> extends
		MemoryEfficientStorageHandler<PHead, T> {

	private final ReentrantReadWriteLock lock = new ReentrantReadWriteLock();
	private final Lock readLock = lock.readLock();
	private final Lock writeLock = lock.writeLock();

	public MemoryEfficientStorageHandlerRWLock(MemoryEfficientAStarAlgorithm<PHead, T> algorithm) {
		super(algorithm);
	}
	

	@Override
	public long getIndexOf(PHead head) throws AStarException {
		readLock.lock();
		try {
			return statespace.contains(new State<PHead, T>(head, null));
		} catch (StorageException e) {
			throw new AStarException(e);
		} finally {
			readLock.unlock();
		}
	}

	@Override
	public void storeStateForRecord(State<PHead, T> state, Record newRec) throws AStarException {
		final Result<State<PHead, T>> r;
		writeLock.lock();
		try {
			r = statespace.add(state);
		} catch (StorageException e) {
			throw new AStarException(e);
		} finally {
			writeLock.unlock();
		}
		newRec.setState(r.index);
	}
}