package org.processmining.plugins.balancedconformance.dataflow.datastate.compression;

import net.jpountz.lz4.LZ4Factory;

import com.google.common.primitives.Longs;

public final class LZ4Compressor implements Compressor {

	private final LZ4Factory factory = LZ4Factory.fastestJavaInstance();

	public long[] decompress(byte[] compressedData, int uncompressedSize) {
		byte[] uncompressedBuffer = new byte[uncompressedSize << 3];
		factory.fastDecompressor().decompress(compressedData, uncompressedBuffer, uncompressedSize * 8);
		long[] data = new long[uncompressedSize];
		for (int j = 0; j < uncompressedSize; j++) {
			int i = j << 3;
			data[j] = Longs.fromBytes(uncompressedBuffer[i], uncompressedBuffer[i + 1], uncompressedBuffer[i + 2],
					uncompressedBuffer[i + 3], uncompressedBuffer[i + 4], uncompressedBuffer[i + 5],
					uncompressedBuffer[i + 6], uncompressedBuffer[i + 7]);
		}
		return data;
	}

	public byte[] compress(long[] uncompressed) {
		byte[] dataOutput = new byte[uncompressed.length << 3];
		int j = 1;
		for (long value : uncompressed) {
			int end = (j << 3) - 1;
			int start = end - 7;
			for (int i = end; i >= start; i--) {
				dataOutput[i] = (byte) (value & 0xffL);
				value >>= 8;
			}
			j++;
		}
		byte[] compressed = factory.fastCompressor().compress(dataOutput);
		//saving.addAndGet((uncompressed.length << 3) - compressed.length);
		return compressed;
	}

}